/*=============================================================================
	OpenGL on windows, manipulating sample @ [get link]

winmain.c
	Windows api code / entry point
=============================================================================*/
#ifndef WINMAIN_C
#define WINMAIN_C
#include "include.h"
#include "opengl.c"

int		newPixelFormat = 0;

int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	MSG		msg;
	HWND		hwnd;
	HDC		hdc;
	HANDLE		hAccel;
	WNDCLASS	wndclass;
	CHAR		szAppName[] = TEXT("Win32OpenGL");

if (!newPixelFormat)
{
	/* Register the frame class */
	wndclass.style		= CS_HREDRAW | CS_VREDRAW;
	wndclass.lpfnWndProc	= WndProc;
	wndclass.cbClsExtra	= 0;
	wndclass.cbWndExtra	= 0;
	wndclass.hInstance	= hInstance;
	wndclass.hIcon		= LoadIcon(hInstance, szAppName);
	wndclass.hCursor	= LoadCursor (NULL, IDC_ARROW);
	wndclass.hbrBackground	= (HBRUSH)(COLOR_BTNFACE + 1);
	wndclass.lpszMenuName	= szAppName;
	wndclass.lpszClassName	= szAppName;

	if (!RegisterClass(&wndclass) )
	{
		MessageBox(NULL, TEXT("Unable to register window class."), szAppName, MB_ICONERROR);
		return FALSE;
	}
}
	/* Create the frame */
	hwnd = CreateWindow(	szAppName,
				szAppName,
				WS_OVERLAPPEDWINDOW,
				CW_USEDEFAULT, CW_USEDEFAULT,
				CW_USEDEFAULT, CW_USEDEFAULT,
				NULL,
				NULL,
				hInstance,
				NULL);

	/* make sure window was created */
	if (!hwnd)
		return FALSE;

	hdc = GetDC(hwnd); 
	SetBkColor(hdc, GetSysColor(COLOR_BTNFACE));
	SetTextColor(hdc, GetSysColor(COLOR_WINDOWTEXT));
	ReleaseDC(hwnd, hdc);
	hAccel = LoadAccelerators(hInstance, szAppName);

	ShowWindow(hwnd, nCmdShow); 
	UpdateWindow(hwnd); 

	/* animation loop */
	while (TRUE)
	{
		while (PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE) == TRUE)
		{
			if (GetMessage(&msg, NULL, 0, 0))
			{
				if (!TranslateAccelerator (hwnd, hAccel, &msg))
				{
					TranslateMessage (&msg) ;
					DispatchMessage (&msg) ;
				}
			} else {
				return TRUE;
			}
		}
		SendMessage(hwnd, WM_IDLE, 0, 0);
	}
	return 0;
}
int nFontList;

LRESULT CALLBACK WndProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	static HDC	hdc		= 0;
	static HMENU	hPopMenu	= 0;
	static HGLRC	hglrc		= 0;		// Handle to opengl rendering context
	static GLfloat	latinc		= LATINC,
			longinc		= LONGINC;
	LONG		lRet = 1;
	RECT		rect;
	POINT		point;

	switch (uMsg)
	{
	case WM_IDLE:
		drawScene(hwnd, hdc);
		break;
	case WM_TIMER:
		updatePos(latinc, longinc);
		break;
	case WM_SIZE:
		GetClientRect(hwnd, &rect);
		resize(rect.right, rect.bottom );
		break;

	case WM_RBUTTONUP:
		point.x = LOWORD(lParam);
		point.y = HIWORD(lParam);
		ClientToScreen(hwnd, &point);
		TrackPopupMenu(hPopMenu, TPM_RIGHTBUTTON, point.x, point.y, 0, hwnd, NULL);
		break;

	case WM_KEYDOWN:
		switch (wParam)
		{
		case VK_LEFT:
			longinc -= 0.5f;
			break;
		case VK_RIGHT:
			longinc += 0.5f;
			break;
		case VK_UP:
			latinc += 0.5f;
			break;
		case VK_DOWN:
			latinc -= 0.5f;
			break;
		}
		break;

	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDM_CLOSE:
			SendMessage(hwnd, WM_CLOSE, 0, 0);
			break;
		case IDM_ABOUT:
			// Modeless seems like a pain in the boot, but probably should be done anyway
			DialogBox((HINSTANCE) GetWindowLong(hwnd, GWL_HINSTANCE), TEXT("about"), hwnd, AboutProc);
			break;
		}
		break;

	case WM_CREATE:
		hPopMenu = LoadMenu((HINSTANCE)GetWindowLong(hwnd, GWL_HINSTANCE), TEXT("PopMenu"));
		hPopMenu = GetSubMenu(hPopMenu, 0);
		hdc = GetDC(hwnd);
		if (!newPixelFormat)
		{
			if (!setupPixelFormat(hdc))			// Takes looong time
				PostQuitMessage (0);
			hglrc = wglCreateContext(hdc);			// create gl rendering context
			wglMakeCurrent(hdc, hglrc);			// set threads current rendering context, dc is "linked" to rc
			if (!setupNewPixelFormat(hwnd, hdc, hglrc))
				MessageBox(NULL, "ChooseNewPixelFormat failed", "Error", MB_OK);
		} else {
			PIXELFORMATDESCRIPTOR pfd;
			pfd.nSize		= sizeof(PIXELFORMATDESCRIPTOR);
			pfd.nVersion		= 1;
			pfd.dwFlags		= PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_GENERIC_ACCELERATED | PFD_DOUBLEBUFFER;
			pfd.dwLayerMask		= PFD_MAIN_PLANE;
			pfd.iPixelType		= PFD_TYPE_RGBA;
			pfd.cColorBits		= 32;
			pfd.cDepthBits		= 16;
			pfd.cAccumBits		= 32;
			pfd.cStencilBits	= 0;
			SetPixelFormat(hdc, newPixelFormat, &pfd);
		}

		wglUseFontBitmaps(hdc, 0, 256, 1000);		// create bitmaps for the device context font's first 256 glyphs 
		GetClientRect(hwnd, &rect);
		initializeGL(rect.right, rect.bottom);
		SetTimer(hwnd, TIMER_TICK, 16, NULL);
		break;

	case WM_CLOSE: 
		DestroyWindow(hwnd);
		break;

	case WM_DESTROY:
		if (hglrc)
		{
			wglMakeCurrent(NULL, NULL);
			wglDeleteContext(hglrc);
			ReleaseDC(hwnd, hdc);
			hglrc = 0;
			hdc = 0;
		}
		glDeleteLists(1000, 256);
		KillTimer (hwnd, TIMER_TICK);
		PostQuitMessage(0);
		break;

	default:
		lRet = DefWindowProc(hwnd, uMsg, wParam, lParam);
		break;
	}
	return lRet;
}

// Legacy way to select format
BOOL setupPixelFormat(HDC hdc)
{
	PIXELFORMATDESCRIPTOR pfd;
	int pixelformat;

	pfd.nSize		= sizeof(PIXELFORMATDESCRIPTOR);
	pfd.nVersion		= 1;
	pfd.dwFlags		= PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_GENERIC_ACCELERATED | PFD_DOUBLEBUFFER;
	pfd.dwLayerMask		= PFD_MAIN_PLANE;
	pfd.iPixelType		= PFD_TYPE_RGBA;
	pfd.cColorBits		= 32;
	pfd.cDepthBits		= 16;
	pfd.cAccumBits		= 32;
	pfd.cStencilBits	= 0;

	if ( (pixelformat = ChoosePixelFormat(hdc, &pfd)) == 0 )
	{
		MessageBox(NULL, "ChoosePixelFormat failed", "Error", MB_OK);
		return FALSE;
	}

	if (SetPixelFormat(hdc, pixelformat, &pfd) == FALSE)
	{
		MessageBox(NULL, "SetPixelFormat failed", "Error", MB_OK);
		return FALSE;
	}
	return TRUE; 
}

/* Must have an existing rendering context in order to get extended pixel format
a window can only change the pixel format once, so we must destroy first window to use
the extended pixel format. Aint life great. */
BOOL setupNewPixelFormat(HWND hwnd, HDC hdc, HGLRC hglrc)
{
	unsigned	numFormats = 10;

	int		iAttributes[] =
			{
				WGL_DRAW_TO_WINDOW_ARB,		GL_TRUE,
				WGL_SUPPORT_OPENGL_ARB,		GL_TRUE,
				WGL_ACCELERATION_ARB,		WGL_FULL_ACCELERATION_ARB, // Specifies an ICD driver. 
				WGL_COLOR_BITS_ARB,		24,
				WGL_ALPHA_BITS_ARB,		8,
				WGL_DEPTH_BITS_ARB,		16,
				WGL_STENCIL_BITS_ARB,		0,
				WGL_DOUBLE_BUFFER_ARB,		GL_TRUE,
				WGL_SAMPLE_BUFFERS_ARB,		GL_TRUE,
				WGL_SAMPLES_ARB,		4,
				0,				0	// terminated by key/value zero
			};
	float	fAttributes[] = { 0, 0 };
	PFNWGLCHOOSEPIXELFORMATARBPROC		wglChoosePixelFormatARB;

	wglChoosePixelFormatARB = (PFNWGLCHOOSEPIXELFORMATARBPROC)wglGetProcAddress("wglChoosePixelFormatARB");
	if (!wglChoosePixelFormatARB)
		return FALSE;
	if (!wglChoosePixelFormatARB(hdc, iAttributes, fAttributes, 1, &newPixelFormat, &numFormats))
		return FALSE;
	// we got our new pixel format, time to destory and recreate
	wglMakeCurrent(NULL, NULL);
	wglDeleteContext(hglrc);
	ReleaseDC(hwnd, hdc);
	hglrc = 0;
	hdc = 0;
	DestroyWindow(hwnd);
	
	return TRUE; 
}

BOOL CALLBACK AboutProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch(message)
	{
	case WM_INITDIALOG:
		return TRUE;
	case WM_COMMAND:
		switch( LOWORD(wParam) )
		{
		case IDOK:
		case IDCANCEL:
			EndDialog(hDlg, 0);
			return TRUE;
		}
		break;
	}
	return FALSE;
}
#endif
