/*=============================================================================
	Win32GL, manipulating sample @ 
[http://msdn.microsoft.com/library/default.asp?
				url=/library/en-us/opengl/oglport_3oqg.asp]
winmain.c
	Windows api code / entry point - test, trying to implement extended pixel format	
					without uglyfying it too much
=============================================================================*/
#ifndef WINMAIN_C
#define WINMAIN_C

#include "include.h"
#include "resource.h"
#include <stdio.h>

LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK WndProcMain(HWND, UINT, WPARAM, LPARAM);
BOOL CALLBACK AboutProc(HWND, UINT, WPARAM, LPARAM);
BOOL setupPixelFormat(HDC);
BOOL setupNewPixelFormat(HDC);

extern void resize(int, int);
extern void initializeGL(int, int);
extern void drawScene(HWND, HDC);
extern void updatePos(double, double);
extern void drawText(char *, float, float);

PFNWGLCHOOSEPIXELFORMATARBPROC	wglChoosePixelFormatARB;

int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	MSG		msg;
	HWND		hwnd;
	HDC		hdc;
	HANDLE		hAccel;
	WNDCLASS	wndclass;
	CHAR		szAppName[] = TEXT("Win32GL");

	/* Register the frame class */
	wndclass.style		= CS_HREDRAW | CS_VREDRAW;
	wndclass.lpfnWndProc	= WndProc;
	wndclass.cbClsExtra	= 0;
	wndclass.cbWndExtra	= 0;
	wndclass.hInstance	= hInstance;
	wndclass.hIcon		= LoadIcon(hInstance, szAppName);
	wndclass.hCursor	= LoadCursor (NULL, IDC_ARROW);
	wndclass.hbrBackground	= NULL;				// Opengl drawing over anyway
//	wndclass.hbrBackground	= (HBRUSH)(COLOR_BTNFACE + 1);
	wndclass.lpszMenuName	= szAppName;
	wndclass.lpszClassName	= "Init";

	if (!RegisterClass(&wndclass) )
	{
		MessageBox(NULL, TEXT("Unable to register window class."), szAppName, MB_ICONERROR);
		return 0;
	}

	/* Create the frame */
	hwnd = CreateWindow(	"Init",
				"Init",
				WS_OVERLAPPEDWINDOW,
				CW_USEDEFAULT, CW_USEDEFAULT,
				CW_USEDEFAULT, CW_USEDEFAULT,
				NULL,
				NULL,
				hInstance,
				NULL);

	/* make sure window was created */
	if (!hwnd)
	{
		MessageBox(NULL, TEXT("Unable to create window."), szAppName, MB_ICONERROR);
		return 0;
	}

	hdc = GetDC(hwnd); 
	SetBkColor(hdc, GetSysColor(COLOR_BTNFACE));
	SetTextColor(hdc, GetSysColor(COLOR_WINDOWTEXT));
	ReleaseDC(hwnd, hdc);
	hAccel = LoadAccelerators(hInstance, szAppName);

	while ( GetMessage(&msg, NULL, 0, 0) )
	{
		TranslateMessage (&msg) ;
		DispatchMessage (&msg) ;
	}

	UnregisterClass("Init", hInstance);
	wndclass.lpfnWndProc	= WndProc;
	wndclass.lpszClassName	= szAppName;

	if (!RegisterClass(&wndclass) )
	{
		MessageBox(NULL, TEXT("Unable to register window class."), szAppName, MB_ICONERROR);
		return 0;
	}

	/* Create the frame */
	hwnd = CreateWindow(	szAppName,
				szAppName,
				WS_OVERLAPPEDWINDOW,
				CW_USEDEFAULT, CW_USEDEFAULT,
				CW_USEDEFAULT, CW_USEDEFAULT,
				NULL,
				NULL,
				hInstance,
				NULL);

	/* make sure window was created */
	if (!hwnd)
	{
		MessageBox(NULL, TEXT("Unable to create window."), szAppName, MB_ICONERROR);
		return 0;
	}

	hdc = GetDC(hwnd); 
	SetBkColor(hdc, GetSysColor(COLOR_BTNFACE));
	SetTextColor(hdc, GetSysColor(COLOR_WINDOWTEXT));
	ReleaseDC(hwnd, hdc);
	hAccel = LoadAccelerators(hInstance, szAppName);

	ShowWindow(hwnd, nCmdShow); 
	UpdateWindow(hwnd); 

	/* animation loop */
	while (TRUE)
	{
		while (PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE) == TRUE)
		{
			GetMessage(&msg, NULL, 0, 0);
			{
				if (!TranslateAccelerator(hwnd, hAccel, &msg))
				{
					TranslateMessage (&msg) ;
					DispatchMessage (&msg) ;
				}
			} else {
				return 0;
			}
		}
		SendMessage(hwnd, WMU_IDLE, 0, 0);
	}
}

LRESULT CALLBACK WndProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	LONG		lRet = 1;
	HDC		hdc;
	HGLRC		hglrc;
	char		*extensionList = NULL;

	switch (uMsg)
	{
	case WM_CREATE:
		hdc = GetDC(hwnd);
		if (!setupPixelFormat(hdc))
			PostQuitMessage (0);
		hglrc = wglCreateContext(hdc);
		wglMakeCurrent(hdc, hglrc);
		extensionList = (char*) glGetString(GL_EXTENSIONS);
		if (strstr(extensionList, "GL_ARB_multisample ") == NULL)
			PostQuitMessage(0);
		wglChoosePixelFormatARB = (PFNWGLCHOOSEPIXELFORMATARBPROC)
						wglGetProcAddress("wglChoosePixelFormatARB");
		if (wglChoosePixelFormatARB == NULL)
			PostQuitMessage(0);
		wglMakeCurrent(NULL, NULL);
		wglDeleteContext(hglrc);
		ReleaseDC(hwnd, hdc);
		PostQuitMessage(0);
		break;
	default:
		lRet = DefWindowProc(hwnd, uMsg, wParam, lParam);
		break;
	}
	return lRet;
}

LRESULT CALLBACK WndProcMain(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	static HDC	hdc		= 0;
	static HMENU	hPopMenu	= 0;
	static HGLRC	hglrc		= 0;		// Handle to opengl rendering context
	static float	latinc		= LATINC,
			longinc		= LONGINC;
	LONG		lRet = 1;
	RECT		rect;
	POINT		point;

	switch (uMsg)
	{
	case WMU_IDLE:
		drawScene(hwnd, hdc);
		break;

	case WM_TIMER:
		updatePos(latinc, longinc);
		break;

	case WM_SIZE:
		GetClientRect(hwnd, &rect);
		resize(rect.right, rect.bottom );
		break;

	case WM_RBUTTONUP:
		point.x = LOWORD(lParam);
		point.y = HIWORD(lParam);
		ClientToScreen(hwnd, &point);
		TrackPopupMenu(hPopMenu, TPM_RIGHTBUTTON, point.x, point.y, 0, hwnd, NULL);
		break;

	case WM_KEYDOWN:
		switch (wParam)
		{
		case VK_LEFT:
			longinc -= 0.5f;
			break;
		case VK_RIGHT:
			longinc += 0.5f;
			break;
		case VK_UP:
			latinc += 0.5f;
			break;
		case VK_DOWN:
			latinc -= 0.5f;
			break;
		}
		break;

	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDM_CLOSE:
			SendMessage(hwnd, WM_CLOSE, 0, 0);
			break;
		case IDM_ABOUT:
			// Modeless seems like a pain in the boot, but probably should be done anyway
			DialogBox((HINSTANCE) GetWindowLong(hwnd, GWL_HINSTANCE), TEXT("about"), hwnd, AboutProc);
			break;
		}
		break;

	case WM_CREATE:
		hPopMenu = LoadMenu((HINSTANCE)GetWindowLong(hwnd, GWL_HINSTANCE), TEXT("PopMenu"));
		hPopMenu = GetSubMenu(hPopMenu, 0);
		hdc = GetDC(hwnd);
		setupNewPixelFormat(hdc);
		hglrc = wglCreateContext(hdc);			// create gl rendering context
		wglMakeCurrent(hdc, hglrc);			// set threads current rendering context, dc is "linked" to rc
		wglUseFontBitmaps(hdc, 0, 256, 1000);		// create bitmaps for the device context font's first 256 glyphs 
		GetClientRect(hwnd, &rect);
		initializeGL(rect.right, rect.bottom);
		SetTimer(hwnd, TIMER_TICK, 16, NULL);
		break;

	case WM_CLOSE: 
		DestroyWindow(hwnd);
		break;

	case WM_DESTROY:
		if (hglrc)
		{
			glDeleteLists(1000, 256);
			wglMakeCurrent(NULL, NULL);
			wglDeleteContext(hglrc);
			ReleaseDC(hwnd, hdc);
			hglrc = 0;
			hdc = 0;
		}
		KillTimer (hwnd, TIMER_TICK);
		PostQuitMessage(0);
		break;

	default:
		lRet = DefWindowProc(hwnd, uMsg, wParam, lParam);
		break;
	}
	return lRet;
}

// Legacy way to select format
BOOL setupPixelFormat(HDC hdc)
{
	PIXELFORMATDESCRIPTOR pfd;
	int pixelformat;

	pfd.nSize		= sizeof(PIXELFORMATDESCRIPTOR);
	pfd.nVersion		= 1;
	pfd.dwFlags		= PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_GENERIC_ACCELERATED | PFD_DOUBLEBUFFER;
	pfd.dwLayerMask		= PFD_MAIN_PLANE;
	pfd.iPixelType		= PFD_TYPE_RGBA;
	pfd.cColorBits		= 16;
	pfd.cDepthBits		= 16;
	pfd.cAccumBits		= 0;
	pfd.cStencilBits	= 0;

	if ( (pixelformat = ChoosePixelFormat(hdc, &pfd)) == 0 )
	{
		MessageBox(NULL, "ChoosePixelFormat failed", "Error", MB_OK);
		return FALSE;
	}

	if (SetPixelFormat(hdc, pixelformat, &pfd) == FALSE)
	{
		MessageBox(NULL, "SetPixelFormat failed", "Error", MB_OK);
		return FALSE;
	}
	return TRUE; 
}

BOOL setupNewPixelFormat(HDC hdc)
{
	int	pixelformat;
	UINT	numFormats;
	float	fAttributes[] = {0,0};

	int iAttributes[] = { WGL_DRAW_TO_WINDOW_ARB,GL_TRUE,
		WGL_SUPPORT_OPENGL_ARB,		GL_TRUE,
		WGL_ACCELERATION_ARB,		WGL_FULL_ACCELERATION_ARB,
		WGL_COLOR_BITS_ARB,		24,
		WGL_ALPHA_BITS_ARB,		8,
		WGL_DEPTH_BITS_ARB,		16,
		WGL_STENCIL_BITS_ARB,		0,
		WGL_DOUBLE_BUFFER_ARB,		GL_TRUE,
		WGL_SAMPLE_BUFFERS_ARB,		GL_TRUE,
		WGL_SAMPLES_ARB,		4,
		0,				0};

	if ( wglChoosePixelFormatARB(hdc, iAttributes, fAttributes, 1, &pixelformat, &numFormats ) == 0)
		PostQuitMessage(0);
	return TRUE;
}

BOOL CALLBACK AboutProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch(message)
	{
	case WM_INITDIALOG:
		return TRUE;
	case WM_COMMAND:
		switch( LOWORD(wParam) )
		{
		case IDOK:
		case IDCANCEL:
			EndDialog(hDlg, 0);
			return TRUE;
		}
		break;
	}
	return FALSE;
}
#endif
