/*=============================================================================
	OpenGL code, keep seperate from windowing code.
opengl.c
	Opengl code
=============================================================================*/
#ifndef OPENGL_C
#define OPENGL_C
#include "include.h"

unsigned getTimeStamp(void);
void resize(int, int);
void initializeGL(int, int);
void drawScene(HWND, HDC);
void polarView(double, double);
void createObjects();
void updatePos(double, double);
void drawText(char *, float, float);
unsigned getClocksPerSec();

unsigned CPUFREQ;

// Called by WM_CREATE
void initializeGL(int width, int height)
{
	double		aspect;
	float		mat_specular[] = { 0.8f, 0.8f, 0.5f, 1.0f };
	float		mat_shininess[] = { 100.0f };
	float		light_position[] = { 1.0f, 1.0f, 1.0f, 0.0f };
	float		fogColor[4] = {0.125f, 0.125f, 0.125f, 1.0f};
	int		fogMode;

	CPUFREQ = getClocksPerSec();
//	glEnable(GL_FOG);
//	glEnable(GL_MULTISAMPLE_ARB);
	fogMode = GL_EXP;
	glFogi(GL_FOG_MODE, fogMode);
	glFogfv(GL_FOG_COLOR, fogColor);
	glFogf(GL_FOG_DENSITY, 0.35f);
	glHint(GL_FOG_HINT, GL_DONT_CARE);

	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);	// Specular hilite
	glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);	// Specular properties?
	glLightfv(GL_LIGHT0, GL_POSITION, light_position);	// Define Light0

	glEnable(GL_LIGHTING);					// Enable Lighting
	glEnable(GL_LIGHT0);					// Enable light0
	glDepthFunc(GL_LEQUAL);

	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);			// Clear color
	glClearDepth(1.0);					// Value used when depth buffer cleared

	glEnable(GL_DEPTH_TEST);				// Z buffer enabled

	glMatrixMode(GL_PROJECTION);
	aspect = (double) width / height;
//	gluPerspective(45.0, aspect, 3.0, 7.0);
	glViewport(0, 0, width, height);
	createObjects();
}

// Called by WMU_IDLE
void drawScene(HWND hwnd, HDC hdc)
{
	unsigned	frameStartTime, frameEndTime, frameElaspedTime;
	char		fps[8];

//	frameStartTime = getTimeStamp();
	frameStartTime = timeGetTime();
	Sleep(1);
	glMatrixMode(GL_MODELVIEW);
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	glCallList(CONE);
	glCallList(CYLINDER);
	glCallList(GLOBE);
//	frameEndTime = getTimeStamp();
	frameEndTime = timeGetTime();
	frameElaspedTime = frameEndTime - frameStartTime;
	if (!frameElaspedTime)					// framerate > 1000
		frameElaspedTime = 1;
//	wsprintf(fps, "%ufps", CPUFREQ / frameElaspedTime);
	wsprintf(fps, "%ufps", 1000 / frameElaspedTime);
	glColor3f( 1.0f, 1.0f, 1.0f );
	drawText(fps, 0.9f, 0.9f);
	glFinish();						// wait till finished so no tearing
	SwapBuffers(hdc);
}

/*
Called by WM_TIMER, keeps animation seperate from drawing so more fps dont speed things up too much.
Small speed bump is due to timer being placed on queue and cpu time needed to get through the queue
*/
void updatePos(double latinc, double longinc)
{
	static double	latitude = 0, longitude = 0;
	static BOOL	pushed = FALSE;

	glMatrixMode(GL_MODELVIEW);
	if (pushed)
		glPopMatrix();
	glPushMatrix();
	glLoadIdentity();
	pushed = TRUE;

        latitude += latinc;
        longitude += longinc;

	glTranslatef(0.0, 0.0, RADIUS);
	glRotated(-TWIST, 0.0, 0.0, 1.0);
	glRotated(-latitude, 1.0, 0.0, 0.0);
	glRotated(longitude, 0.0, 0.0, 1.0);
}

// lists are like macros, nothing actually drawn / affected here
void createObjects()
{
	GLUquadricObj *quadObj;

	glColorMaterial(GL_FRONT, GL_DIFFUSE);
	glNewList(GLOBE, GL_COMPILE);
		glEnable(GL_COLOR_MATERIAL);
		glColor3f( 0.0f, 0.0f, 0.5f );
		quadObj = gluNewQuadric();
		gluQuadricDrawStyle(quadObj, GLU_LINE);
		gluSphere(quadObj, 1.5, 16, 16);
		glDisable(GL_COLOR_MATERIAL);
	glEndList();
 
	glNewList(CONE, GL_COMPILE);
		glEnable(GL_COLOR_MATERIAL);
		glColor3f( 0.0f, 0.5f, 0.0f );

		quadObj = gluNewQuadric();
		gluQuadricDrawStyle(quadObj, GLU_FILL);
		gluQuadricNormals(quadObj, GLU_SMOOTH);
		gluCylinder(quadObj, 0.3, 0.0, 0.6, 30, 20);

		glDisable(GL_COLOR_MATERIAL);
	glEndList();
 
	glNewList(CYLINDER, GL_COMPILE);
		gluQuadricTexture(quadObj, GL_TRUE);
		glEnable(GL_COLOR_MATERIAL);					// Enable colored lighting
		glColor3f( 0.5f, 0.0f, 0.0f );
		glPushMatrix();
			glTranslatef(0.8f, -0.65f, 0.0f);
			glRotatef(30.0f, 1.0f, 0.5f, 1.0f);
			glPushMatrix();							// Save current matrix, changes affect only what is drawn immediately
				glRotatef(90.0, (double)1.0, (double)0.0, (double)0.0);		// Rotates: angle xyz
				glTranslatef((double)0.0, (double)0.0, (double)-0.5);		// Translates(slides): xyz
				quadObj = gluNewQuadric();					// New quad
				gluQuadricDrawStyle(quadObj, GLU_FILL);				// Style
				gluQuadricNormals(quadObj, GLU_SMOOTH);				// Smooth shaded
				gluCylinder(quadObj, 0.3, 0.3, 0.6, 24, 2);			// Draw cylinder: baseRadius, topRadius, height, slices, stacks
			glPopMatrix();						// Restore matrix
		glPopMatrix();
		glDisable(GL_COLOR_MATERIAL);					// Disable colored lighting
	glEndList();
}

void resize( int width, int height )
{
	glViewport(0, 0, width, height);
	glMatrixMode(GL_PROJECTION);			// Select matrix stack [GL_PROJECTION | GL_MODELVIEW | GL_TEXTURE ]
	glLoadIdentity();				// Load blank matrix
	gluPerspective( 45.0, (double) width / height, 3.0, 7.0 );	// gluPerspective( GLdouble fovy, GLdouble aspect, GLdouble zNear, GLdouble zFar );
}

void drawText(char *string, float x, float y)
{
	glDisable(GL_LIGHTING);
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
		glLoadIdentity();
		glOrtho (0.0f, 1.0f, 0.0f, 1.0f, 1.0f, 0.0f);
		glMatrixMode(GL_MODELVIEW);
		glPushMatrix();
			glLoadIdentity();
			glRasterPos2f(x, y);					// xy text position scale 0.0 to 1.0
			glListBase(1000);					// set up for a string-drawing display list call 
			glCallLists(lstrlen(string), GL_UNSIGNED_BYTE, string);	// draw a string using font display lists	
		glPopMatrix();
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glEnable(GL_LIGHTING);					// Enable Lighting
}

unsigned getTimeStamp(void)
{
	unsigned timestamp = 0;

	_asm
	{
		// rdtsc returns 64bit "timestamp" in edx:eax, timestamp is really a count of clock cycles
		rdtsc
		// we are only interested in small time intervals, so highword is worthless,
		mov	DWORD PTR timestamp, eax
	}
	return timestamp;
}

unsigned getClocksPerSec()
{
	unsigned time;

	time = getTimeStamp();
	Sleep(1000);
	return getTimeStamp() - time;
}
#endif
