/*=============================================================================
	Win32GL, manipulating sample @ 
[http://msdn.microsoft.com/library/default.asp?
				url=/library/en-us/opengl/oglport_3oqg.asp]
winmain.c
	Windows api code / entry point
=============================================================================*/
/*
	DEVMODE		dmScreenSettings;

	memset(&dmScreenSettings, 0, sizeof(dmScreenSettings));
	dmScreenSettings.dmSize		= sizeof(dmScreenSettings);
	dmScreenSettings.dmPelsWidth	= 1024; 
	dmScreenSettings.dmPelsHeight	= 768;
	dmScreenSettings.dmBitsPerPel	= 32;
	dmScreenSettings.dmFields	= DM_BITSPERPEL | DM_PELSWIDTH | DM_PELSHEIGHT;

	ChangeDisplaySettings(&dmScreenSettings, CDS_FULLSCREEN);
*/

#ifndef WINMAIN_C
#define WINMAIN_C

#include "include.h"
#include "resource.h"
//#pragma comment(lib, "opengl32.lib")

LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
int windowMain(HINSTANCE, HINSTANCE, LPSTR, int);
BOOL CALLBACK AboutProc(HWND, UINT, WPARAM, LPARAM);
BOOL CALLBACK SettingsProc(HWND, UINT, WPARAM, LPARAM);
BOOL setupPixelFormat(HDC);
BOOL setupNewPixelFormat(HWND, HDC, HGLRC);

void resize(int, int);
void initializeGL(int, int);
void drawScene(HWND, HDC);
void updatePos(double, double);
void drawText(char *, float, float);

int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	windowMain(hInstance, hPrevInstance, lpCmdLine, nCmdShow);
	return 0;
}

int windowMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	MSG		msg;
	HWND		hwnd;
	HDC		hdc;
	HANDLE		hAccel;
	WNDCLASS	wndclass;
	TCHAR		szAppName[]	= TEXT("Win32GL");
	int		running		= 1;

	wndclass.style		= CS_HREDRAW | CS_VREDRAW;
	wndclass.lpfnWndProc	= WndProc;
	wndclass.cbClsExtra	= 0;
	wndclass.cbWndExtra	= 0;
	wndclass.hInstance	= hInstance;
	wndclass.hIcon		= LoadIcon(hInstance, szAppName);
	wndclass.hCursor	= LoadCursor (NULL, IDC_ARROW);
	wndclass.hbrBackground	= NULL;				// Opengl drawing over anyway
//	wndclass.hbrBackground	= (HBRUSH)(COLOR_BTNFACE + 1);
	wndclass.lpszMenuName	= szAppName;
	wndclass.lpszClassName	= szAppName;

	if (!RegisterClass(&wndclass) )
	{
		MessageBox(NULL, TEXT("Unable to register window class."), szAppName, MB_ICONERROR);
		return 1;
	}

	hwnd = CreateWindow(	szAppName,
				szAppName,
				WS_OVERLAPPEDWINDOW,
				CW_USEDEFAULT, CW_USEDEFAULT,
				CW_USEDEFAULT, CW_USEDEFAULT,		// should match current res for fullscreen, default otherwise
				NULL,
				NULL,
				hInstance,
				NULL);

	if (!hwnd)
	{
		MessageBox(NULL, TEXT("Unable to create window."), szAppName, MB_ICONERROR);
		return 1;
	}

	hdc = GetDC(hwnd); 
	SetBkColor(hdc, GetSysColor(COLOR_BTNFACE));
	SetTextColor(hdc, GetSysColor(COLOR_WINDOWTEXT));
	ReleaseDC(hwnd, hdc);
	hAccel = LoadAccelerators(hInstance, szAppName);

	SetForegroundWindow(hwnd);	// forces our window on top of others "fullscreen" toggle
	ShowWindow(hwnd, nCmdShow); 
	UpdateWindow(hwnd); 

	while (running)
	{
		while (PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))
		{
			if ( GetMessage(&msg, NULL, 0, 0) > 0 )
			{
				if (!TranslateAccelerator(hwnd, hAccel, &msg))
				{
					TranslateMessage (&msg) ;
					DispatchMessage (&msg) ;
				}
			} else {
				running = 0;
			}
		}
		SendMessage(hwnd, UWM_IDLE, 0, 0);
	}

	if (!UnregisterClass(szAppName, hInstance))
	{
		MessageBox(NULL, TEXT("Unable to unregister class."), szAppName, MB_ICONERROR);
		return 1;
	}
	return 0;
}

LRESULT CALLBACK WndProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	static HDC	hdc		= 0;
	static HMENU	hPopMenu	= 0;
	static HGLRC	hglrc		= 0;		// Handle to opengl rendering context
	static float	latinc		= LATINC,
			longinc		= LONGINC;
	LONG		lRet = 1;
	RECT		rect;
	POINT		point;

	switch (uMsg)
	{
	case UWM_IDLE:
		drawScene(hwnd, hdc);
		break;

	case WM_TIMER:
		updatePos(latinc, longinc);
		break;

	case WM_SIZE:
		GetClientRect(hwnd, &rect);
		resize(rect.right, rect.bottom );
		break;

	case WM_RBUTTONUP:
		point.x = LOWORD(lParam);
		point.y = HIWORD(lParam);
		ClientToScreen(hwnd, &point);
		TrackPopupMenu(hPopMenu, TPM_RIGHTBUTTON, point.x, point.y, 0, hwnd, NULL);
		break;

	case WM_KEYDOWN:
		switch (wParam)
		{
		case VK_LEFT:
			longinc -= 0.5f;
			break;
		case VK_RIGHT:
			longinc += 0.5f;
			break;
		case VK_UP:
			latinc += 0.5f;
			break;
		case VK_DOWN:
			latinc -= 0.5f;
			break;
		}
		break;

	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDM_SETTINGS:
			DialogBox((HINSTANCE) GetWindowLong(hwnd, GWL_HINSTANCE), TEXT("settings"), hwnd, SettingsProc);
			break;
		case IDM_ABOUT:
			// Modeless seems like a pain in the boot, but probably should be done anyway
			DialogBox((HINSTANCE) GetWindowLong(hwnd, GWL_HINSTANCE), TEXT("about"), hwnd, AboutProc);
			break;
		case IDM_CLOSE:
			SendMessage(hwnd, WM_CLOSE, 0, 0);
			break;
		}
		break;

	case WM_CREATE:
		hPopMenu = LoadMenu((HINSTANCE)GetWindowLong(hwnd, GWL_HINSTANCE), TEXT("PopMenu"));
		hPopMenu = GetSubMenu(hPopMenu, 0);
		hdc = GetDC(hwnd);
		if (!setupPixelFormat(hdc))			// Takes looong time
			PostQuitMessage (0);
		hglrc = wglCreateContext(hdc);			// create gl rendering context
		wglMakeCurrent(hdc, hglrc);			// set threads current rendering context, dc is "linked" to rc
		wglUseFontBitmaps(hdc, 0, 256, 1000);		// create bitmaps for the device context font's first 256 glyphs 
		GetClientRect(hwnd, &rect);
		initializeGL(rect.right, rect.bottom);
		SetTimer(hwnd, TIMER_TICK, 16, NULL);
		break;

	case WM_CLOSE: 
		DestroyWindow(hwnd);
		break;

	case WM_DESTROY:
		ChangeDisplaySettings(NULL, 0);
		if (hglrc)
		{
			glDeleteLists(1000, 256);
			wglMakeCurrent(NULL, NULL);
			wglDeleteContext(hglrc);
			ReleaseDC(hwnd, hdc);
			hglrc = 0;
			hdc = 0;
		}
		KillTimer (hwnd, TIMER_TICK);
		PostQuitMessage(0);
		break;

	default:
		lRet = DefWindowProc(hwnd, uMsg, wParam, lParam);
		break;
	}
	return lRet;
}

// Legacy way to select format
BOOL setupPixelFormat(HDC hdc)
{
	PIXELFORMATDESCRIPTOR pfd;
	int pixelformat;

	pfd.nSize		= sizeof(PIXELFORMATDESCRIPTOR);
	pfd.nVersion		= 1;
	pfd.dwFlags		= PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_GENERIC_ACCELERATED | PFD_DOUBLEBUFFER;
	pfd.dwLayerMask		= PFD_MAIN_PLANE;
	pfd.iPixelType		= PFD_TYPE_RGBA;
	pfd.cColorBits		= 16;
	pfd.cDepthBits		= 16;
	pfd.cAccumBits		= 0;
	pfd.cStencilBits	= 0;

	if ( (pixelformat = ChoosePixelFormat(hdc, &pfd)) == 0 )
	{
		MessageBox(NULL, "ChoosePixelFormat failed", "Error", MB_OK);
		return FALSE;
	}

	if (SetPixelFormat(hdc, pixelformat, &pfd) == FALSE)
	{
		MessageBox(NULL, "SetPixelFormat failed", "Error", MB_OK);
		return FALSE;
	}
	return TRUE; 
}

BOOL CALLBACK AboutProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch(message)
	{
	case WM_INITDIALOG:
		return TRUE;
	case WM_COMMAND:
		switch( LOWORD(wParam) )
		{
		case IDOK:
		case IDCANCEL:
			EndDialog(hDlg, 0);
			return TRUE;
		}
		break;
	}
	return FALSE;
}

BOOL CALLBACK SettingsProc(HWND hdlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	static HWND	hwndCombo = 0;
	DEVMODE		dmScreenSettings;
	TCHAR		resbuf[80];
	static TCHAR	currentRes[80];
	int i;

	switch(message)
	{
	case WM_INITDIALOG:
		hwndCombo = GetDlgItem(hdlg, IDD_RESOLUTION);

		// populate available resolutions
		for (i = 1; i != 0; i++)
		{
			if ( EnumDisplaySettings(NULL, i - 1,  &dmScreenSettings) )
			{
				wsprintf(resbuf, "%dx%d %dbpp",	dmScreenSettings.dmPelsWidth,
								dmScreenSettings.dmPelsHeight,
								dmScreenSettings.dmBitsPerPel);
				// add to combo box
				SendMessage(hwndCombo, CB_ADDSTRING, 0, (LPARAM) resbuf);
			} else {
				i = -1;
			}
		}

		// make default selection in dropdown box
		EnumDisplaySettings(NULL, ENUM_CURRENT_SETTINGS,  &dmScreenSettings);
		wsprintf(currentRes, "%dx%d %dbpp",	dmScreenSettings.dmPelsWidth,
							dmScreenSettings.dmPelsHeight,
							dmScreenSettings.dmBitsPerPel);
		SendMessage (hwndCombo, CB_SELECTSTRING, -1, (LPARAM) currentRes);
		return TRUE;
	case WM_COMMAND:
		switch( LOWORD(wParam) )
		{
		case IDD_RESOLUTION:
			return TRUE;
		case IDOK:
			if ( SendMessage (GetDlgItem(hdlg, IDD_FULLSCREEN), BM_GETCHECK, 0, 0) )
			{
				// check res and go fullscreen
				memset(&dmScreenSettings, 0, sizeof(dmScreenSettings));
				dmScreenSettings.dmSize	= sizeof(dmScreenSettings);
				dmScreenSettings.dmFields = DM_BITSPERPEL | DM_PELSWIDTH | DM_PELSHEIGHT;
				SendMessage (hwndCombo, LB_GETTEXT, SendMessage (hwndCombo, CB_GETCURSEL, 0, 0), (LPARAM)currentRes);
/*				sscanf(currentRes, "%dx%x %dbpp",	&(dmScreenSettings.dmPelsWidth),
									&(dmScreenSettings.dmPelsHeight),
									&(dmScreenSettings.dmBitsPerPel));*/
				ChangeDisplaySettings(&dmScreenSettings, CDS_FULLSCREEN);
			}
			EndDialog(hdlg, 0);
			return TRUE;
		case IDCANCEL:
			EndDialog(hdlg, 0);
			return TRUE;
		}
		break;
	}
	return FALSE;
}
#endif
